﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class TroopArmy : MonoBehaviour
{
	public TroopGrid grid;

	public int faction = 0; // All the troops I create will have this faction #.
	public GameObject troop;
	public GameObject enemy;

	public int ARMY_X = 10;
	public int ARMY_Z = 10;


	public bool formation = false;
	
	List<Troop> my_troops;

	void Start ()
	{
		my_troops = new List<Troop>();

		for(int i=0;i<ARMY_X;i++)
		{
			Troop troop;
			for(int j=0;j<ARMY_Z;j++)
			{
				Vector3 pos;
				if(faction==0)
				{
					pos = new Vector3(i*grid.SPACING_X,1.5f,(j+1)*grid.SPACING_Z);

				}
				else
				{
					pos = new Vector3(100f - (i+1)*grid.SPACING_X,1.5f,(j+1)*grid.SPACING_Z);
					//Debug.Log (pos);
				}

				troop = CreateTroop(pos);
				if(faction==0) // Redundant if. TODO: Remove it
				{
					//troop.SetNode(j,i);
					troop.name = "Troop " + troop.node_x + "_" + j;
				}
				else
				{
					//troop.SetNode(grid.GRID_SIZE_X-j,i);
					troop.name = "Troop " + troop.node_x + "_" + j;
				}

			}
		}
	}



	Troop CreateTroop(Vector3 pos)
	{
		GameObject new_troop = Instantiate(troop,pos,transform.rotation) as GameObject;

//		Move troop_script = new_troop.GetComponent<Move>();
//		troop_script.target = enemy.transform;
//		troop_script.faction = faction;

		/*
		if(faction==1) // Flip
		{
			Vector3 temp = new_troop.transform.localScale;
			temp.x *= -1;
			new_troop.transform.localScale = temp;
		}
		*/

		Troop troop_script = new_troop.GetComponent<Troop>();
		troop_script.faction = faction;
		troop_script.army = this;
		my_troops.Add(troop_script);

		int x = (int)(pos.x/grid.SPACING_X);
		int z = (int)(pos.z/grid.SPACING_Z);
		grid.PutSoldierAtNode(troop_script,x,z);

		return troop_script;
	}

	public void SetOrders(int new_order)
	{
		switch(new_order)
		{
			case 0: SetOrders(ORDERS.HOLD); break;
			case 3: SetOrders(ORDERS.ADVANCE); break;
			case 4: SetOrders(ORDERS.RANGED); break;
			case 5: SetOrders(ORDERS.CHARGE); break;
			default: break;
		}
	}

	public void SetOrders(ORDERS new_order)
	{
		foreach(Troop s in my_troops)
		{
			s.orders = new_order;
		}
	}

	/// <summary>
	/// Remove a soldier from my list, so that it's never commanded or targeted again.
	/// </summary>
	/// <param name="casualty">Casualty.</param>
	public void TellMeSoldierWasKilled(Troop casualty)
	{
		my_troops.Remove(casualty);
	}

	/// <summary>
	/// Called by an _enemy_ of this army, to find someone on _this_ side.
	/// </summary>
	public Troop FindAnEnemyToAttack(Troop attacker, float range)
	{
		// WIP.

		Vector3 attacker_coords = attacker.transform.position;

		float closest_distance;
		Troop closest_enemy = attacker; // Default
		foreach(Troop soldier in my_troops)
		{
			float distance = Vector3.Distance(attacker_coords,soldier.transform.position) + Random.Range(0f,2f);
			if(distance < range)
			{
				closest_distance = distance;
				closest_enemy = soldier;
			}
		}
		return closest_enemy;
	}


}
